@@ -13,7 +13,6 @@ from PyFin.Analysis.SecurityValueHolders import SecurityLatestValueHolder
 from PyFin.Analysis.CrossSectionValueHolders import CSRankedSecurityValueHolder
 from PyFin.Analysis.CrossSectionValueHolders import CSAverageSecurityValueHolder
 from PyFin.Analysis.CrossSectionValueHolders import CSAverageAdjustedSecurityValueHolder
-from PyFin.Analysis.CrossSectionValueHolders import CSQuantileSecurityValueHolder
 from PyFin.Analysis.CrossSectionValueHolders import CSZScoreSecurityValueHolder
 from PyFin.Analysis.CrossSectionValueHolders import CSPercentileSecurityValueHolder
 from PyFin.Analysis.CrossSectionValueHolders import CSResidueSecurityValueHolder
@@ -96,22 +95,83 @@ class TestCrossSectionValueHolder(unittest.TestCase):
             benchmark.push(data)
             meanHolder.push(data)
             benchmarkValues = benchmark.value
-            np.testing.assert_array_almost_equal(benchmarkValues.mean(), meanHolder.value.values)
+            np.testing.assert_array_almost_equal(benchmarkValues.values.mean(), meanHolder.value.values)
+
+    def testCSAverageSecurityValueHolderWithGroup(self):
+        benchmark = SecurityLatestValueHolder(x='close')
+        groups = SecurityLatestValueHolder(x='ind')
+        meanHolder = CSAverageSecurityValueHolder(benchmark, groups)
+
+        for i in range(len(self.datas['aapl']['close'])):
+            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
+                             'ind': 1.},
+                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
+                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
+                            'ind': 1.},
+                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
+                             'ind': 2.},
+                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
+                             'ind': 2.}}
+            benchmark.push(data)
+            meanHolder.push(data)
+            benchmarkValues = benchmark.value
+            groups = {'aapl': 1., 'ibm': 1., 'goog': 2., 'baba': 2.}
+            expected_mean = pd.Series(benchmarkValues.to_dict()).groupby(groups).mean()
+            calculated_mean = meanHolder.value
+
+            for name in calculated_mean.index():
+                if name in ['aapl', 'ibm']:
+                    self.assertAlmostEqual(calculated_mean[name], expected_mean[1])
+                else:
+                    self.assertAlmostEqual(calculated_mean[name], expected_mean[2])
 
     def testCSPercentileSecurityValueHolder(self):
-        percent = 50
         benchmark = SecurityLatestValueHolder(x='close')
-        perHolder = CSPercentileSecurityValueHolder(percent, benchmark)
+        perHolder = CSPercentileSecurityValueHolder(benchmark)
 
         for i in range(len(self.datas['aapl']['close'])):
             data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                              Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                     'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
-                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]}}
+                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]},
+                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i]},
+                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
+                            Factors.OPEN: self.datas['baba'][Factors.OPEN][i]}
+                    }
             benchmark.push(data)
             perHolder.push(data)
             benchmarkValues = benchmark.value
-            np.testing.assert_array_almost_equal(np.percentile(benchmarkValues.values, 50), perHolder.value.values)
+            np.testing.assert_array_almost_equal(benchmarkValues.rank().values / (len(data) - 1), perHolder.value.values)
+
+    def testCSPercentileSecurityValueHolderWithGroups(self):
+        benchmark = SecurityLatestValueHolder(x='close')
+        groups = SecurityLatestValueHolder(x='ind')
+        perHolder = CSPercentileSecurityValueHolder(benchmark, groups)
+
+        for i in range(len(self.datas['aapl']['close'])):
+            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
+                             'ind': 1.},
+                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
+                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
+                            'ind': 1.},
+                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
+                             'ind': 2.},
+                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
+                             'ind': 2.}}
+            benchmark.push(data)
+            perHolder.push(data)
+            benchmarkValues = benchmark.value
+            groups = {'aapl': 1., 'ibm': 1., 'goog': 2., 'baba': 2.}
+            expected_rank = pd.Series(benchmarkValues.to_dict()).groupby(groups) \
+                .transform(lambda x: (x.rank().values - 1.) / (len(x) - 1))
+            np.testing.assert_array_almost_equal(expected_rank, perHolder.value.values)
 
     def testCSAverageAdjustedSecurityValueHolder(self):
         benchmark = SecurityLatestValueHolder(x='close')
@@ -121,32 +181,38 @@ class TestCrossSectionValueHolder(unittest.TestCase):
             data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                              Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                     'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
-                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]}}
+                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]},
+                    }
             benchmark.push(data)
             meanAdjustedHolder.push(data)
             benchmarkValues = benchmark.value
-            np.testing.assert_array_almost_equal(benchmarkValues.values - benchmarkValues.mean(), meanAdjustedHolder.value.values)
+            np.testing.assert_array_almost_equal((benchmarkValues - benchmarkValues.mean()).values, meanAdjustedHolder.value.values)
 
-    def testCSQuantileSecurityValueHolder(self):
-        keys = list(range(1, 11))
-        values = list(range(10, 0, -1))
+    def testCSAverageAdjustedSecurityValueHolderWithGroups(self):
+        benchmark = SecurityLatestValueHolder(x='close')
+        groups = SecurityLatestValueHolder(x='ind')
+        meanAdjustedHolder = CSAverageAdjustedSecurityValueHolder(benchmark, groups)
 
-        data = {}
-
-        for i, k in enumerate(keys):
-            data[k] = {}
-            data[k]['close'] = values[i]
-
-        quantile_value = CSQuantileSecurityValueHolder('close')
-        quantile_value.push(data)
-
-        calculated = quantile_value.value
-
-        data = np.linspace(1., 0., 10)
-
-        expected = pd.Series(data=data, index=[x for x in range(0, 10)])
-
-        np.testing.assert_array_almost_equal(expected.values, calculated.values)
+        for i in range(len(self.datas['aapl']['close'])):
+            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
+                             'ind': 1.},
+                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
+                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
+                            'ind': 1.},
+                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
+                             'ind': 2.},
+                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
+                             'ind': 2.}}
+            benchmark.push(data)
+            meanAdjustedHolder.push(data)
+            benchmarkValues = benchmark.value
+            groups = {'aapl': 1., 'ibm': 1., 'goog': 2., 'baba': 2.}
+            expected_rank = pd.Series(benchmarkValues.to_dict()).groupby(groups) \
+                .transform(lambda x: x - x.mean())
+            np.testing.assert_array_almost_equal(expected_rank, meanAdjustedHolder.value.values)
 
     def testCSZscoreSecurityValueHolder(self):
         keys = list(range(1, 11))
@@ -168,6 +234,32 @@ class TestCrossSectionValueHolder(unittest.TestCase):
 
         np.testing.assert_array_almost_equal(expected, calculated.values)
 
+    def testCSZscoreSecurityValueHolderWithGroups(self):
+        benchmark = SecurityLatestValueHolder(x='close')
+        groups = SecurityLatestValueHolder(x='ind')
+        meanAdjustedHolder = CSZScoreSecurityValueHolder(benchmark, groups)
+
+        for i in range(len(self.datas['aapl']['close'])):
+            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
+                             'ind': 1.},
+                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
+                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
+                            'ind': 1.},
+                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
+                             'ind': 2.},
+                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
+                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
+                             'ind': 2.}}
+            benchmark.push(data)
+            meanAdjustedHolder.push(data)
+            benchmarkValues = benchmark.value
+            groups = {'aapl': 1., 'ibm': 1., 'goog': 2., 'baba': 2.}
+            expected_rank = pd.Series(benchmarkValues.to_dict()).groupby(groups) \
+                .transform(lambda x: (x - x.mean()) / x.std(ddof=0))
+            np.testing.assert_array_almost_equal(expected_rank, meanAdjustedHolder.value.values)
+
     def testCSZResidueSecurityValueHolder(self):
         y = SecurityLatestValueHolder(x='close')
         x = SecurityLatestValueHolder(x='open')
